/*==============================================================================
    FILE:           user_adau1979_1962a.c

    DESCRIPTION:    Provides a interface between the user_audio USB Audio 2.0
                    functionality and the ADI adau1979 and adau1962a drivers.

    Copyright (c) 2018 Closed Loop Design, LLC

    This software is supplied "AS IS" without any warranties, express, implied
    or statutory, including but not limited to the implied warranties of fitness
    for purpose, satisfactory quality and non-infringement. Closed Loop Design LLC
    extends you a royalty-free right to use, reproduce and distribute this source
    file as well as executable files created using this source file for use with 
    Analog Devices SC5xx family processors only. Nothing else gives you 
    the right to use this source file.

==============================================================================*/
/*!
 * @file      user_adau1979_1962a.c
 * @brief     Interface logic for the ADAU1979 and ADAU1962a drivers.
 *
 * @details
 *            Provides an interface between the user_audio USB Audio 2.0
 *            functionality and the ADI adau1979 and adau1962a drivers.
 *
 *            Copyright (c) 2018 Closed Loop Design, LLC
 *
 *            This software is supplied "AS IS" without any warranties, express, implied
 *            or statutory, including but not limited to the implied warranties of fitness
 *            for purpose, satisfactory quality and non-infringement. Closed Loop Design LLC
 *            extends you a royalty-free right to use, reproduce and distribute this source
 *            file as well as executable files created using this source file for use with 
 *            Analog Devices SC5xx family processors only. Nothing else gives you 
 *            the right to use this source file.
 *
 */

#include "user_adau1979_1962a.h"

#include "ll_mgr.h"
#include <services/spu/adi_spu.h>
#include <services/gpio/adi_gpio.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include "cld_sc58x_audio_2_0_w_cdc_lib.h"
#include "user_audio_w_cdc.h"

/* ADAU1979 SPORT config parameters */
#define LR_B_CLK_MASTER_1979    (true)
#define BCLK_RISING_1979        (true)
#define LRCLK_HI_LO_1979        (false)

/* DAC Master clock frequency */
#if (USER_AUDIO_48kHz_SAMPLE_RATE)
#define ADAU1962A_MCLK_IN       (24576000u)
/* DAC sample rate */
#define SAMPLE_RATE             (48000u)
#define SAMPLE_RATE_IN_HZ        48000
#else
#define ADAU1962A_MCLK_IN       (22579200u)
/* DAC sample rate */
#define SAMPLE_RATE             (44100u)
#define SAMPLE_RATE_IN_HZ        44100
#endif

/* ADAU1962A SPORT config parameters */
#define LR_B_CLK_MASTER_1962    (true)
#define BCLK_RISING_1962        (false)
#define LRCLK_HI_LO_1962        (false)

#define USB_POLLING_INTERVAL_MICROSECONDS       1000
#if (USER_AUDIO_48kHz_SAMPLE_RATE)
#define NUM_AUDIO_SAMPLES                       12
#else
#define NUM_AUDIO_SAMPLES                       12
#endif
#define AUDIO_SAMPLE_SIZE_BYTES                 4


#define USER_ADAU1979_1962A_BUFFER_SIZE               (NUM_AUDIO_SAMPLES*AUDIO_SAMPLE_SIZE_BYTES*NUM_AUDIO_CHANNELS)

/* Sport System Protection Init IDs */
#define SPORT_4A_SPU_PID            (23u)
#define SPORT_4B_SPU_PID            (24u)
#define SPORT_4A_DMA10_SPU_PID      (74u)
#define SPORT_4B_DMA11_SPU_PID      (75u)


static LL_Mgr_Data dac_ll;
static LL_Mgr_Data adc_ll;

unsigned long TwiMemory[ADI_TWI_MEMORY_SIZE];           /*!< Analog Devices Twi System Services memory */

/* ADAU1979 ADC */
static ADI_ADAU1979_HANDLE phAdau1979;                  /*!< Analog Devices adau1979 System Services handle */
unsigned long Adau1979Memory[ADI_ADAU1979_MEMORY_SIZE]; /*!< Analog Devices adau1979 System Services memory */
unsigned long Adau1979SportMemory[ADI_SPORT_DMA_MEMORY_SIZE];   /*!< Analog Devices adau1979 Sport System Services memory */

/* ADAU1962A DAC */
static ADI_ADAU1962A_HANDLE phAdau1962a;                    /*!< Analog Devices adau1962a System Services handle */
unsigned long Adau1962aMemory[ADI_ADAU1962A_MEMORY_SIZE];   /*!< Analog Devices adau1962a System Services memory */
unsigned long Adau1962aSportMemory[ADI_SPORT_DMA_MEMORY_SIZE];  /*!< Analog Devices adau1962a Sport System Services memory */

static unsigned char  SpuMemory[ADI_SPU_MEMORY_SIZE];   /*!< Analog Devices SPU System Services memory */
static ADI_SPU_HANDLE hSpu;                             /*!< Analog Devices SPU System Services handle */

static void user_adau1979_check_adi_result(ADI_ADAU1979_RESULT result);
static void user_adau1962a_check_adi_result(ADI_ADAU1962A_RESULT result);
static void user_adau1979_1962a_tx_callback(void *pCBParam, uint32_t Event, void *pArg);
static void user_adau1979_1962a_rx_callback(void *pCBParam, uint32_t Event, void *pArg);
static unsigned char user_adau1979_convert_usb_volume_to_codec (unsigned short usb_vol);
static unsigned char user_adau1962a_convert_usb_volume_to_codec (unsigned short usb_vol);
static unsigned short user_adau1962a_convert_volume_to_usb (unsigned char adau1962a_vol);
static unsigned short user_adau1979_convert_volume_to_usb (unsigned char adau1979_vol);
static unsigned long Adau1979Init(void);
static unsigned long Adau1962aInit(void);
static void Adau1979SubmitBuffers(void);
static void Adau1962aSubmitBuffers(void);
static unsigned long SpuInit(void);

static unsigned char *p_available_adau1979_1962a_audio_data_rx_buffer = NULL; /*!< Pointer to the available ADAU1962A ping-pong buffer. */

static unsigned char user_adau1979_1962a_submit_rx_buffer = 0; /*!< Flag used to notify mainline an adau1962a ping-pong
                                                                          buffer should be resubmitted to the Sport DMA */
static unsigned char user_adau1979_1962a_submit_tx_buffer = 0; /*!< Flag used to notify mainline an adau1979 ping-pong
                                                                          buffer should be resubmitted to the Sport DMA */


unsigned short active_adau1979_1962a_audio_data_tx_buffer_num_bytes = USER_ADAU1979_1962A_BUFFER_SIZE; /*!< Number of bytes in the active Tx ping-pong buffer */

static unsigned char adau1979_1962a_audio_data_tx_cnt = 0;

static CLD_Boolean user_adau_init_error;                /*!< Flag used to notify if an error occurred initializing the CODECs */

static CLD_Boolean user_adau_input_enabled = CLD_FALSE; /*!< Flag used to track if the adau1979 input is enabled */
static CLD_Boolean user_adau_output_enabled = CLD_FALSE;/*!< Flag used to track if the adau1962a output is enabled */

/**
 * Checks if the ADAU1979 result passed to the function is an error.
 *
 * @param result - ADAU1979 result code.
 */
static void user_adau1979_check_adi_result(ADI_ADAU1979_RESULT result)
{
    if (result != ADI_ADAU1979_SUCCESS)
    {
        printf("1979 Codec failure 0x%x\n\r", result);
        user_adau_init_error = CLD_TRUE;
    }
}

/**
 * Checks if the ADAU1962a result passed to the function is an error
 *
 * @param result - ADAU1962a result code.
 */
static void user_adau1962a_check_adi_result(ADI_ADAU1962A_RESULT result)
{
    if (result != ADI_ADAU1962A_SUCCESS)
    {
        printf("1962a Codec failure 0x%x\n\r", result);
        user_adau_init_error = CLD_TRUE;
    }
}

/**
 * Initializes the ADAU1979 and ADAC1962A CODECs.
 *
 * @retval USER_ADAU1979_1962a_INIT_SUCCESS - Initialization successful
 * @retval USER_ADAU1979_1962a_INIT_FAILED  - Initialization failed
 */
User_ADAU1979_1962a_Init_Return_Code user_adau1979_1962a_init (void)
{
    unsigned long rv = 0;

    user_adau_init_error = CLD_FALSE;

    ll_mgr_init(&dac_ll);
    ll_mgr_init(&adc_ll);

    /* Initialize SPU */
    rv = SpuInit();

    if (rv == 0)
    {
        rv = Adau1979Init();
    }
    if (rv == 0)
    {
        rv = Adau1962aInit();
    }
    /* Submit ADC buffers */
    if(rv == 0u)
    {
        Adau1979SubmitBuffers();
    }

    /* Submit DAC buffers */
    if(rv == 0u)
    {
        Adau1962aSubmitBuffers();
    }

    /* Enable data flow for only the ADC and DAC */
    if(rv == 0u)
    {
        if((unsigned long)adi_adau1962a_Enable(phAdau1962a, true) != 0u)
        {
            rv = 1;
        }
    }

    if(rv == 0u)
    {
        if((unsigned long)adi_adau1979_Enable(phAdau1979, true) != 0u)
        {
            /* return error */
            rv =  1u;
        }
    }

    if (rv)
    {
        user_adau_init_error = CLD_TRUE;
    }

    if (user_adau_init_error == CLD_TRUE)
    {
        return USER_ADAU1979_1962a_INIT_FAILED;
    }
    else
    {
        return USER_ADAU1979_1962a_INIT_SUCCESS;
    }
}

/**
 * Mainline function used to interface with the ADAU1979 and ADAC1962A CODECs.
 *
 */
void user_adau1979_1962a_main (void)
{
    static unsigned char input_enabled = 0xaa;
    static unsigned char output_enabled = 0xaa;

    ADI_ADAU1962A_RESULT result;
    ADI_ADAU1979_RESULT  result_2;
    LL_Mgr_Node * p_nn;

    /* If the ADAU1962A output has been enabled/disabled */
    if (output_enabled != user_adau_output_enabled)
    {
        output_enabled = user_adau_output_enabled;

        if (user_adau_output_enabled == CLD_TRUE)
        {
            /* Disable audio output */
            result = adi_adau1962a_Enable (phAdau1962a, false);
            user_adau1962a_check_adi_result(result);

            user_adau1979_1962a_submit_tx_buffer = 0;

            Adau1962aSubmitBuffers();

            /* Enable audio output */
            result = adi_adau1962a_Enable (phAdau1962a, true);
            user_adau1962a_check_adi_result(result);
        }
        else
        {
            /* Disable audio output */
            result = adi_adau1962a_Enable (phAdau1962a, false);
            user_adau1962a_check_adi_result(result);

            ll_mgr_reset(&dac_ll);
        }
    }

    /* If the ADAU1979 input has been enabled/disabled */
    if (input_enabled != user_adau_input_enabled)
    {
        input_enabled = user_adau_input_enabled;

        if (user_adau_input_enabled == CLD_TRUE)
        {
            /* Disable microphone input */
            result_2 = adi_adau1979_Enable (phAdau1979, false);
            user_adau1979_check_adi_result(result_2);

            user_adau1979_1962a_submit_rx_buffer = 0;

            Adau1979SubmitBuffers();

            /* Enable input */
            result_2 = adi_adau1979_Enable (phAdau1979, true);
            user_adau1979_check_adi_result(result_2);

            adau1979_1962a_audio_data_tx_cnt = 0;
        }
        else
        {
            /* Disable audio input */
            result_2 = adi_adau1979_Enable (phAdau1979, false);
            user_adau1979_check_adi_result(result_2);

            ll_mgr_reset(&adc_ll);
        }
    }

    /* Re-submit the processed ADAU1979 buffer to the Sport DMA if the USB audio input is enabled */
    if (user_adau1979_1962a_submit_rx_buffer && user_adau_input_enabled == CLD_TRUE)
    {
        user_adau1979_1962a_submit_rx_buffer--;

        p_nn = ll_mgr_remove(&adc_ll);

        /* Pass the received data to the USB Audio. */
        #if (USER_AUDIO_LOOPBACK == 0)
        if (p_nn)
        {
            user_audio_transmit_audio_stream_data (p_nn);
        }
        #else
        if (p_nn)
        {
            ll_mgr_free_node (p_nn);
        }
        #endif
    }

    /* Re-submit the processed ADAU1962A buffer to the Sport DMA if the USB audio output is enabled */
    if (user_adau1979_1962a_submit_tx_buffer && user_adau_output_enabled == CLD_TRUE)
    {
        user_adau1979_1962a_submit_tx_buffer--;
    }
}

/**
 * Function called when the ADAU1979 ADC Sport data has been received.
 *
 * @param pCBParam - Callback parameters pointer
 * @param Event - Event that triggered this callback function.
 * @param pArg - Pointer to the data received by the Sport.
 */
static void user_adau1979_1962a_rx_callback(void *pCBParam, uint32_t Event, void *pArg)
{
    ADI_ADAU1979_RESULT  result_2;
    unsigned short adc_length;
    LL_Mgr_Node * p_nn;

    /* If the USB Audio input is enabled */
    if (user_adau_input_enabled == CLD_TRUE)
    {
        /* If Audio input data has been received. */
        if (Event == (unsigned long)ADI_SPORT_EVENT_RX_BUFFER_PROCESSED)
        {
            /* Save a pointer to the buffer so it can be resubmitted. */
            p_available_adau1979_1962a_audio_data_rx_buffer = pArg;

            /* Flag that the buffer is ready to be resubmitted */
            user_adau1979_1962a_submit_rx_buffer++;

            adau1979_1962a_audio_data_tx_cnt++;


#if (USER_AUDIO_48kHz_SAMPLE_RATE)

            adc_length = USER_ADAU1979_1962A_BUFFER_SIZE;
#else
            if (adau1979_1962a_audio_data_tx_cnt >= 40)
            {
                adau1979_1962a_audio_data_tx_cnt = 0;
                adc_length = USER_ADAU1979_1962A_BUFFER_SIZE;
            }
            else
            {
                adc_length = USER_ADAU1979_1962A_BUFFER_SIZE-(AUDIO_SAMPLE_SIZE_BYTES*NUM_AUDIO_CHANNELS);
            }
#endif
#if (USER_AUDIO_VARIABLE_FEEDBACK)
            if (user_audio_audio_in_get_num_queued_packets() >= 2)
            {
                adc_length += (AUDIO_SAMPLE_SIZE_BYTES*NUM_AUDIO_CHANNELS);
            }
#endif

            p_nn = ll_mgr_get_node(0);
            if (p_nn)
            {
                ll_mgr_insert(&adc_ll, p_nn);
                p_nn->size = adc_length;
                result_2 = adi_adau1979_SubmitBuffer(phAdau1979, &p_nn->data, adc_length);
                user_adau1979_check_adi_result(result_2);
            }
        }
        else
        {
            printf("Rx Sport Error 0x%x\n\r", (unsigned int)Event);
        }
    }
    else
    {
    }
}

/**
 * Function called when the ADAU1962a DAC Sport data has been transmitted.
 *
 * @param pCBParam - Callback parameters pointer
 * @param Event - Event that triggered this callback function.
 * @param pArg - Pointer to the transmitted data buffer.
 */
static void user_adau1979_1962a_tx_callback(void *pCBParam, uint32_t Event, void *pArg)
{
    LL_Mgr_Node * p_nn;
    ADI_ADAU1962A_RESULT  result;

    /* If USB output is enabled */
    if (user_adau_output_enabled == CLD_TRUE)
    {
        /* If USB audio data has been transmitted to the ADAU1962a codec. */
        if (Event == (unsigned long)ADI_SPORT_EVENT_TX_BUFFER_PROCESSED)
        {
            /* Flag that the buffer is ready to be resubmitted */
            user_adau1979_1962a_submit_tx_buffer++;


            p_nn = ll_mgr_remove(&dac_ll);
            ll_mgr_free_node(p_nn);


            p_nn = user_audio_w_cdc_get_audio_out_data();

            if (p_nn == 0)
            {
                p_nn = ll_mgr_get_node(CLD_TRUE);
                if (p_nn)
                {
#if (USER_AUDIO_48kHz_SAMPLE_RATE)
                    p_nn->size = USER_ADAU1979_1962A_BUFFER_SIZE;
#else
                    p_nn->size = USER_ADAU1979_1962A_BUFFER_SIZE - (AUDIO_SAMPLE_SIZE_BYTES*NUM_AUDIO_CHANNELS);
#endif
                }
            }
            if (p_nn)
            {
                ll_mgr_insert(&dac_ll, p_nn);
                result = adi_adau1962a_SubmitBuffer(phAdau1962a, &p_nn->data, p_nn->size);
                user_adau1962a_check_adi_result(result);
            }
        }
        else
        {
            printf("Sport event 0x%x\n\r", (unsigned int)Event);
        }
    }
}

/**
 * Sets the volume for the specified ADAU1762A channel.
 *
 * @param eChannel - ADAU1962A DAC channel.
 * @param usb_volume - Volume set by the USB Host.
 */
void user_adau1962a_set_volume_headphones (
    ADI_ADAU1962A_CHNL                   eChannel,
    unsigned short                 const usb_volume)
{
    ADI_ADAU1962A_RESULT result;

    /* Convert the requested USB volume to the Codec volume and call the
       ADI set volume function. */
    result = adi_adau1962a_SetVolume (phAdau1962a, eChannel, user_adau1962a_convert_usb_volume_to_codec(usb_volume));

    user_adau1962a_check_adi_result(result);
}

/**
 * Sets the volume for the specified ADAU1979 channel.
 *
 * @param eChannel - ADAU1979 DAC channel.
 * @param usb_volume - Volume set by the USB Host.
 */
void user_adau1979_set_volume_mic (
    ADI_ADAU1979_AUDIO_CHANNEL                   eChannel,
    unsigned short                 const usb_volume)
{
    ADI_ADAU1979_RESULT result;

    /* Convert the requested USB volume to the Codec volume and call the
       ADAU set volume function. */
    result = adi_adau1979_SetVolume (phAdau1979, eChannel, user_adau1979_convert_usb_volume_to_codec(usb_volume));

    user_adau1979_check_adi_result(result);
}

/**
 * Function used to mute/unmute an ADAU1962A DAC channel.
 *
 * @param eChannel - ADAU1979 DAC channel.
 * @param mute - non-zero = Mute, zero = mute.
 */
void user_adau1962a_set_mute_headphones (
    ADI_ADAU1962A_CHNL                   eChannel,
    unsigned char                         mute)
{
    ADI_ADAU1962A_RESULT result;
    result = adi_adau1962a_MuteChannel(phAdau1962a, eChannel, (mute ? true : false));
    user_adau1962a_check_adi_result(result);
}

/**
 * Function used to mute/unmute an ADAU1979 ADC channel.
 *
 * @param eChannel - ADAU1979 DAC channel.
 * @param mute - non-zero = Mute, zero = mute.
 */
void user_adau1979_set_mute_mic (
    ADI_ADAU1979_AUDIO_CHANNEL            eChannel,
    unsigned char                         mute)
{
    ADI_ADAU1979_RESULT result;
    result = adi_adau1979_MuteChannel(phAdau1979, eChannel, (mute ? true : false));
    user_adau1962a_check_adi_result(result);
}

/**
 * Function used get the current volume of the specified ADAU1962A channel using
 * the USB Audio format.
 *
 * @param eChannel - ADAU1962A DAC channel.
 * @param p_volume - Pointer to the variable to store the current volume level.
 */
void user_adau1962a_get_volume_headphones (
    ADI_ADAU1962A_CHNL                  eChannel,
    unsigned short *                    p_volume)
{
    static const ADI_ADAU1962A_REG channel_registers[] =
    {
        ADI_ADAU1962A_REG_DAC1_VOL,       /* DAC  1 Volume control register */
        ADI_ADAU1962A_REG_DAC2_VOL,       /* DAC  2 Volume control register */
        ADI_ADAU1962A_REG_DAC3_VOL,       /* DAC  3 Volume control register */
        ADI_ADAU1962A_REG_DAC4_VOL,       /* DAC  4 Volume control register */
        ADI_ADAU1962A_REG_DAC5_VOL,       /* DAC  5 Volume control register */
        ADI_ADAU1962A_REG_DAC6_VOL,       /* DAC  6 Volume control register */
        ADI_ADAU1962A_REG_DAC7_VOL,       /* DAC  7 Volume control register */
        ADI_ADAU1962A_REG_DAC8_VOL,       /* DAC  8 Volume control register */
        ADI_ADAU1962A_REG_DAC9_VOL,       /* DAC  9 Volume control register */
        ADI_ADAU1962A_REG_DAC10_VOL,      /* DAC 10 Volume control register */
        ADI_ADAU1962A_REG_DAC11_VOL,      /* DAC 11 Volume control register */
        ADI_ADAU1962A_REG_DAC12_VOL,      /* DAC 12 Volume control register */
        ADI_ADAU1962A_REG_DACMSTR_VOL,    /* DAC Master Volume control register */
    };

    ADI_ADAU1962A_RESULT result;

    unsigned char volume;


    result = adi_adau1962a_ReadRegister (phAdau1962a, channel_registers[eChannel], &volume);
    *p_volume = user_adau1962a_convert_volume_to_usb(volume);

    user_adau1962a_check_adi_result(result);
}

/**
 * Function used get the current volume of the specified ADAU1979 channel using
 * the USB Audio format.
 *
 * @param eChannel - ADAU1979 DAC channel.
 * @param p_volume - Pointer to the variable to store the current volume level.
 */
void user_adau1979_get_volume_mic (
    ADI_ADAU1979_AUDIO_CHANNEL      eChannel,
    unsigned short *                p_volume)
{
    static const ADI_ADAU1979_REG channel_registers[] =
    {
        ADI_ADAU1979_REG_POSTADC_GAIN1,  /* Gain control for channel 1 */
        ADI_ADAU1979_REG_POSTADC_GAIN2,  /* Gain control for channel 2 */
        ADI_ADAU1979_REG_POSTADC_GAIN3,  /* Gain control for channel 3 */
        ADI_ADAU1979_REG_POSTADC_GAIN4,  /* Gain control for channel 4 */
    };

    ADI_ADAU1979_RESULT result;

    unsigned char volume;

    result = adi_adau1979_ReadRegister (phAdau1979, channel_registers[eChannel], &volume);
    *p_volume = user_adau1979_convert_volume_to_usb(volume);

    user_adau1962a_check_adi_result(result);
}

/**
 * Function used get the current mute setting of the specified ADAU1962A channel.
 *
 * @param eChannel - ADAU1962A DAC channel.
 * @retval 0 = not muted
 * @retval 1 = muted.
 */
unsigned char user_adau1962a_get_mute (
        ADI_ADAU1962A_CHNL                   eChannel)
{
    ADI_ADAU1962A_RESULT result = ADI_ADAU1962A_FAILURE;

    unsigned char mute;

    if (eChannel <= ADI_ADAU1962A_CHNL_DAC_8)
    {
        result = adi_adau1962a_ReadRegister (phAdau1962a, ADI_ADAU1962A_REG_DAC_MUTE1, &mute);
        if ((mute & (1 << eChannel)) == 0)
        {
            return 0;
        }
        else
        {
            return 1;
        }
    }
    else if (eChannel <= ADI_ADAU1962A_CHNL_DAC_12)
    {
        result = adi_adau1962a_ReadRegister (phAdau1962a, ADI_ADAU1962A_REG_DAC_MUTE2, &mute);
        if ((mute & (1 << (eChannel - ADI_ADAU1962A_CHNL_DAC_9))) == 0)
        {
            return 0;
        }
        else
        {
            return 1;
        }
    }
    else if (eChannel == ADI_ADAU1962A_CHNL_DAC_MSTR)
    {
        result = adi_adau1962a_ReadRegister (phAdau1962a, ADI_ADAU1962A_REG_DAC_MUTE1, &mute);
        if ((mute & 0x03) == 0)
        {
            return 0;
        }
        else
        {
            return 1;
        }
    }
    user_adau1962a_check_adi_result(result);

    return 0;
}

/**
 * Function used get the current mute setting of the specified ADAU1979 channel.
 *
 * @param eChannel - ADAU1962A DAC channel.
 * @retval 0 = not muted
 * @retval 1 = muted.
 */
unsigned char user_adau1979_get_mute_mic (
    ADI_ADAU1979_AUDIO_CHANNEL                   eChannel)
{
    static const ADI_ADAU1979_REG channel_registers[] =
    {
        ADI_ADAU1979_REG_POSTADC_GAIN1,  /*!< Gain control for channel 1 */
        ADI_ADAU1979_REG_POSTADC_GAIN2,  /*!< Gain control for channel 2 */
        ADI_ADAU1979_REG_POSTADC_GAIN3,  /*!< Gain control for channel 3 */
        ADI_ADAU1979_REG_POSTADC_GAIN4,  /*!< Gain control for channel 4 */
    };

    ADI_ADAU1979_RESULT result;

    unsigned char mute;


    result = adi_adau1979_ReadRegister (phAdau1979, channel_registers[eChannel], &mute);
    if (mute == 0xff)
    {
        mute = 1;
    }
    else
    {
        mute = 0;
    }
    user_adau1962a_check_adi_result(result);

    return mute;
}

/**
 * Converts the passed ADAU1962A volume setting to the equivalent
 * USB Audio v2.0 setting
 *
 * The ADAU1962A volume is a 8-bit value that ranges from -95.626dB to 0dB
 *          in 0.375dB increments.
 *          The USB volume is a 8-bit value ranging from -127.
 *          ranges from -127.9961 to 127.9961 in 1/256 dB increments
 *
 * @param adau1962a_vol - ADAU1962A volume setting.
 * @retval USB Audio volume setting
 */
static unsigned short user_adau1962a_convert_volume_to_usb (unsigned char adau1962a_vol)
{
    signed short usb_vol;

    if (adau1962a_vol == 0)
    {
        usb_vol = 0;
    }
    else
    {
        usb_vol = ((adau1962a_vol * 0.375 * -1))/(1.0/256.0);
    }
    return (unsigned short)usb_vol;
}

/**
 * Converts the passed ADAU1979 volume setting to the equivalent
 * USB Audio v2.0 setting
 *
 * The ADAU1979 volume is a 8-bit value that ranges from -35.625dB to 60dB
 *          in 0.375dB increments.
 *          The USB volume is a 8-bit value ranging from -127.
 *          ranges from -127.9961 to 127.9961 in 1/256 dB increments
 *
 * @param adau1979_vol - ADAU1979 volume setting.
 * @retval USB Audio volume setting
 */
static unsigned short user_adau1979_convert_volume_to_usb (unsigned char adau1979_vol)
{
    unsigned short usb_vol;

    if (adau1979_vol == 0xa0)
    {
        usb_vol = 0;
    }
    else
    {
        usb_vol = ((0xa0 - adau1979_vol) * 0.375) / (1.0/256.0);
    }

    return usb_vol;
}

/**
 * Converts the passed USB volume setting to the to the equivalent
 * ADAU1962A volume setting.
 *
 * @param usb_vol - USB volume setting.
 * @retval ADAU1962A volume setting
 */
static unsigned char user_adau1962a_convert_usb_volume_to_codec (unsigned short usb_vol)
{
    signed short adj_usb;
    unsigned char codec_vol;

    if (usb_vol == 0)
    {
        codec_vol = 0;
    }
    else
    {
        adj_usb = (signed short)usb_vol;
        adj_usb *= -1;
        codec_vol = (adj_usb / 96);
    }
    return codec_vol;
}

/**
 * Converts the passed USB volume setting to the to the equivalent
 * ADAU1979 volume setting.
 *
 * @param usb_vol - USB volume setting.
 * @retval ADAU1979 volume setting
 */
static unsigned char user_adau1979_convert_usb_volume_to_codec (unsigned short usb_vol)
{
    /* adj_usb is now from -127.9961 to 127.9961 */
    float adj_usb = ((signed short)usb_vol) * ((float)1/(float)256);
    unsigned short codec_vol;

    /* adau1979 ranges from +60 to -35.625 in 0.375 increments */
    if (adj_usb == 0)
    {
        codec_vol = 0xa0;
    }
    else if (adj_usb < 0)
    {
        adj_usb *= 1;
        adj_usb /= 0.375;
        adj_usb = 0xa0 + adj_usb;
        if (adj_usb > 0xff)
        {
            codec_vol = 0xff;
        }
        else
        {
            codec_vol = (unsigned char) adj_usb;
        }
    }
    else
    {
        adj_usb /= 0.375;
        adj_usb = 0xa0 - adj_usb;
        if (adj_usb < 0)
        {
            codec_vol = 0;
        }
        else
        {
            codec_vol = (unsigned char) adj_usb;
        }
    }

    return codec_vol;
}

/**
 * Enables the ADAU1962A DAC output.
 */
void user_adau1962a_enable_output (void)
{
#if (USER_AUDIO_LOOPBACK == 0)
    user_adau_output_enabled = CLD_TRUE;
#endif
}

/**
 * Disables the ADAU1962A DAC output.
 */
void user_adau1962a_disable_output (void)
{
    user_adau_output_enabled = CLD_FALSE;
}

/**
 * Enables the ADAU1979 ADC input.
 */
void user_adau1979_enable_input (void)
{
#if (USER_AUDIO_LOOPBACK == 0)
    user_adau_input_enabled = CLD_TRUE;
#endif
}

/**
 * Disables the ADAU1979 ADC input.
 */
void user_adau1979_disable_input (void)
{
    user_adau_input_enabled = CLD_FALSE;
}

/**
 * Opens and initializes ADAU1979 ADC Device.
 *
 * This function was copied, and then modified, from the ADI
 * examples provided with the SC589 EZ-Board Board Support Package.
 *
 * @retval 0 if success
 * @retval other values for error
 */
static unsigned long Adau1979Init(void)
{
    unsigned long Result = 0u;
    /* Instance to submit SPORT configuration */
    ADI_ADAU1979_SPORT_CONFIG   SportConfig;
    /* Instance to submit TWI configuration */
    ADI_ADAU1979_TWI_CONFIG     TwiConfig;

    /* open ADAU1979 instance */
    if((uint32_t)adi_adau1979_Open(0u,
                                   ADI_ADAU1979_SERIAL_MODE_TDM8,
                                   &Adau1979Memory,
                                   ADI_ADAU1979_MEMORY_SIZE,
                                   &phAdau1979) != 0u)
    {
        printf ("ADAU1979: adi_adau1979_Open failed\n");
        /* return error */
        return 1u;
    }

    /* TWI parameters required to open/configure TWI */
    TwiConfig.TwiDevNum     = 0u;
    TwiConfig.TwiDevMemSize = ADI_TWI_MEMORY_SIZE;
    TwiConfig.pTwiDevMem    = &TwiMemory;
    TwiConfig.eTwiAddr      = ADI_ADAU1979_TWI_ADDR_11;

    if((uint32_t)adi_adau1979_ConfigTwi(phAdau1979, &TwiConfig) != 0u)
    {
        printf ("ADAU1979: adi_adau1979_ConfigTwi failed\n");
        /* return error */
        return 1u;
    }

    /* SPORT parameters required to open/configure TWI */
#if defined(__ADSPBF707_FAMILY__) || defined(__ADSPSC589_FAMILY__)
    SportConfig.SportDevNum     = 4u;
#elif defined(__ADSPSC573_FAMILY__)
    SportConfig.SportDevNum     = 2u;
#else
#error "processor not defined"
#endif
    SportConfig.SportDevMemSize = ADI_SPORT_DMA_MEMORY_SIZE;
    SportConfig.pSportDevMem    = &Adau1979SportMemory;
    SportConfig.eSportChnl      = ADI_ADAU1979_SPORT_A;
    SportConfig.eSportPri       = ADI_ADAU1979_SERIAL_PORT_DSDATA1;
    SportConfig.eSportSec       = ADI_ADAU1979_SERIAL_PORT_NONE;
    SportConfig.bLsbFirst       = true;

    if((uint32_t)adi_adau1979_ConfigSport(phAdau1979, &SportConfig) != 0u)
    {
        printf ("ADAU1979: adi_adau1979_ConfigSport failed\n");
        /* return error */
        return 1u;
    }

    /* ADC is a master source of SPORT clk and FS, MCLK 25.576 MHz and PLL used MCLK */
#if (USER_AUDIO_48kHz_SAMPLE_RATE)
    if((uint32_t)adi_adau1979_ConfigPllClk(phAdau1979,
                                           LR_B_CLK_MASTER_1979,
                                           ADI_ADAU1979_MCLK_IN_FREQ_24576000HZ,
                                           ADI_ADAU1979_PLL_SOURCE_MCLK) != 0u)
#else
        if((uint32_t)adi_adau1979_ConfigPllClk(phAdau1979,
                                                   LR_B_CLK_MASTER_1979,
                                                   ADI_ADAU1979_MCLK_IN_FREQ_22579200HZ,
                                                   ADI_ADAU1979_PLL_SOURCE_MCLK) != 0u)
#endif
    {
        printf ("ADAU1979: adi_adau1979_ConfigPllClk failed\n");
        /* return error */
        return 1u;
    }

    if((uint32_t)adi_adau1979_ConfigSerialClk(phAdau1979,
                                              BCLK_RISING_1979,
                                              LRCLK_HI_LO_1979) != 0u)
    {
        printf ("ADAU1979: adi_adau1979_ConfigSerialClk failed\n");
        /* return error */
        return 1u;
    }
#if (USER_AUDIO_48kHz_SAMPLE_RATE)
    if((uint32_t)adi_adau1979_SetSampleRate(phAdau1979, ADI_ADAU1979_SAMPLE_RATE_48000HZ) != 0u)
#else
    if((uint32_t)adi_adau1979_SetSampleRate(phAdau1979, ADI_ADAU1979_SAMPLE_RATE_44100HZ) != 0u)
#endif
    {
        printf ("ADAU1979: adi_adau1979_SetSampleRate failed\n");
        /* return error */
        return 1u;
    }
    if((uint32_t)adi_adau1979_SetWordWidth(phAdau1979, ADI_ADAU1979_WORD_WIDTH_24) != 0u)
    {
        printf ("ADAU1979: adi_adau1979_SetWordWidth failed\n");
        /* return error */
        return 1u;
    }

    if((uint32_t)adi_adau1979_SetTdmTimeslotChannelMap(phAdau1979, ADI_ADAU1979_AUDIO_CHANNEL1,ADI_ADAU1979_TDM_TIMESLOT_02) != 0u)
    {
        printf ("ADAU1979: adi_adau1979_SetTdmTimeslotChannelMap 1 failed\n");
        /* return error */
        return 1u;
    }
    if((uint32_t)adi_adau1979_SetTdmTimeslotChannelMap(phAdau1979, ADI_ADAU1979_AUDIO_CHANNEL2,ADI_ADAU1979_TDM_TIMESLOT_01) != 0u)
    {
        printf ("ADAU1979: adi_adau1979_SetTdmTimeslotChannelMap 2 failed\n");
        /* return error */
        return 1u;
    }

    if((uint32_t)adi_adau1979_SetTdmTimeslotChannelMap(phAdau1979, ADI_ADAU1979_AUDIO_CHANNEL3,ADI_ADAU1979_TDM_TIMESLOT_04) != 0u)
    {
        printf ("ADAU1979: adi_adau1979_SetTdmTimeslotChannelMap 3 failed\n");
        /* return error */
        return 1u;
    }
    if((uint32_t)adi_adau1979_SetTdmTimeslotChannelMap(phAdau1979, ADI_ADAU1979_AUDIO_CHANNEL4,ADI_ADAU1979_TDM_TIMESLOT_03) != 0u)
    {
        printf ("ADAU1979: adi_adau1979_SetTdmTimeslotChannelMap 4 failed\n");
        /* return error */
        return 1u;
    }

    if((uint32_t)adi_adau1979_RegisterCallback(phAdau1979, user_adau1979_1962a_rx_callback, NULL) != 0u)
    {
        printf ("ADAU1979: adi_adau1979_RegisterCallback failed\n");
        /* return error */
        return 1u;
    }

    return Result;
}

/**
 * Opens and initializes ADAU1962A DAC Device.
 *
 * This function was copied, and then modified, from the ADI
 * examples provided with the SC589 EZ-Board Board Support Package.
 *
 * @retval 0 if success
 * @retval other values for error
 */
static unsigned long Adau1962aInit(void)
{
    ADI_ADAU1962A_RESULT        eResult;
    ADI_ADAU1962A_TWI_CONFIG    TwiConfig;
    ADI_ADAU1962A_SPORT_CONFIG  SportConfig;

    /* Open ADAU1962A device instance */
    if((eResult = adi_adau1962a_Open(0u,
                                     ADI_ADAU1962A_SERIAL_MODE_TDM8,
                                     &Adau1962aMemory,
                                     ADI_ADAU1962A_MEMORY_SIZE,
                                     &phAdau1962a)) != ADI_ADAU1962A_SUCCESS)
    {
        printf ("ADAU1962A: Failed to open ADAU1962A device instance, Error Code: 0x%08X\n", eResult);
        /* return error */
        return 1u;
    }

    /* TWI parameters required to open/configure TWI */
    TwiConfig.TwiDevNum     = 0u;
    TwiConfig.eTwiAddr      = ADI_ADAU1962A_TWI_ADDR_04;
    TwiConfig.TwiDevMemSize = ADI_TWI_MEMORY_SIZE;
    TwiConfig.pTwiDevMem    = &TwiMemory;

    /* Configure TWI */
    if ((eResult = adi_adau1962a_ConfigTwi (phAdau1962a, &TwiConfig)) != ADI_ADAU1962A_SUCCESS)
    {
        printf ("ADAU1962A: Failed to configure TWI, Error Code: 0x%08X\n", eResult);
        /* return error */
        return 1u;
    }

    /* SPORT parameters required to open/configure SPORT */
#if defined(__ADSPBF707_FAMILY__) || defined(__ADSPSC589_FAMILY__)
    SportConfig.SportDevNum     = 4u;
#elif defined(__ADSPSC573_FAMILY__)
    SportConfig.SportDevNum     = 2u;
#else
#error "processor not defined"
#endif
    SportConfig.eSportChnl      = ADI_ADAU1962A_SPORT_B;
    SportConfig.eSportPri       = ADI_ADAU1962A_SERIAL_PORT_DSDATA1;
    SportConfig.eSportSec       = ADI_ADAU1962A_SERIAL_PORT_NONE;
    SportConfig.SportDevMemSize = ADI_SPORT_DMA_MEMORY_SIZE;
    SportConfig.pSportDevMem    = &Adau1962aSportMemory;

    /* Configure SPORT */
    if ((eResult = adi_adau1962a_ConfigSport (phAdau1962a, &SportConfig)) != ADI_ADAU1962A_SUCCESS)
    {
        printf ("ADAU1962A: Failed to configure SPORT, Error Code: 0x%08X\n", eResult);
        /* return error */
        return 1u;
    }

    /* DAC Master Power-up */
    if ((eResult = adi_adau1962a_ConfigDacPwr (phAdau1962a,
                                               ADI_ADAU1962A_CHNL_DAC_MSTR,
                                               ADI_ADAU1962A_DAC_PWR_LOW,
                                               true)) != ADI_ADAU1962A_SUCCESS)
    {
        printf ("ADAU1962A: Failed to configure DAC power, Error Code: 0x%08X\n", eResult);
        /* return error */
        return 1u;
    }

    /*
     * Configure PLL clock - DAC is clock master and drives SPORT clk and FS
     * MCLK 24.576 MHz and PLL uses MCLK
     */
    if ((eResult = adi_adau1962a_ConfigPllClk (phAdau1962a,
                                               ADAU1962A_MCLK_IN,
                                               ADI_ADAU1962A_MCLK_SEL_PLL,
                                               ADI_ADAU1962A_PLL_IN_MCLKI_XTALI)) != ADI_ADAU1962A_SUCCESS)
    {
        printf ("ADAU1962A: Failed to configure PLL clock, Error Code: 0x%08X\n", eResult);
        /* return error */
        return 1u;
    }

    /*
     * Configure serial data clock
     * DAC as clock master, External BCLK, Latch on raising edge
     * LRCLK at 50% duty cycle, MSB first, Left channel at LRCLK low
     */
    if ((eResult = adi_adau1962a_ConfigSerialClk (phAdau1962a,
                                                  LR_B_CLK_MASTER_1962,
                                                  false,
                                                  BCLK_RISING_1962,
                                                  true,
                                                  false,
                                                  LRCLK_HI_LO_1962)) != ADI_ADAU1962A_SUCCESS)
    {
        printf ("ADAU1962A: Failed to configure serial data clock, Error Code: 0x%08X\n", eResult);
        /* return error */
        return 1u;
    }

    /* Power-up PLL */
    if ((eResult = adi_adau1962a_ConfigBlockPwr (phAdau1962a,
                                                 false,
                                                 true,
                                                 true)) != ADI_ADAU1962A_SUCCESS)
    {
        printf ("ADAU1962A: Failed to Power-up PLL, Error Code: 0x%08X\n", eResult);
        /* return error */
        return 1u;
    }

    /* Configure Sample rate */
    if ((eResult = adi_adau1962a_SetSampleRate (phAdau1962a, SAMPLE_RATE)) != ADI_ADAU1962A_SUCCESS)
    {
        printf ("ADAU1962A: Failed to configure Sample rate, Error Code: 0x%08X\n", eResult);
        /* return error */
        return 1u;
    }

    /* Configure Word width */
    if ((eResult = adi_adau1962a_SetWordWidth (phAdau1962a,
                                               31)) != ADI_ADAU1962A_SUCCESS)
                                               //ADI_ADAU1962A_WORD_WIDTH_24)) != ADI_ADAU1962A_SUCCESS)
    {
        printf ("ADAU1962A: Failed to configure word width, Error Code: 0x%08X\n", eResult);
        /* return error */
        return 1u;
    }

    /* Register callback */
    if ((eResult = adi_adau1962a_RegisterCallback (phAdau1962a,
                                                   user_adau1979_1962a_tx_callback,
                                                   NULL)) != ADI_ADAU1962A_SUCCESS)
    {
        printf ("ADAU1962A: Failed to register callback, Error Code: 0x%08X\n", eResult);
        /* return error */
        return 1u;
    }

    return 0u;
}

/**
 * Submits the ADAU1979 ping pong buffers to the ADI Sport System Service.
 *
 * This function was copied, and then modified, from the ADI
 * examples provided with the SC589 EZ-Board Board Support Package.
 *
 * @retval 0 if success
 * @retval other values for error
 */
static void Adau1979SubmitBuffers(void)
{
    LL_Mgr_Node * p_nn;
    ADI_ADAU1979_RESULT result;

    /* submit buffer1 */
#if (USER_AUDIO_48kHz_SAMPLE_RATE)
    p_nn = ll_mgr_get_node(0);
    if (p_nn)
    {
        ll_mgr_insert(&adc_ll, p_nn);
        p_nn->size = USER_ADAU1979_1962A_BUFFER_SIZE;
        result = adi_adau1979_SubmitBuffer(phAdau1979, &p_nn->data, USER_ADAU1979_1962A_BUFFER_SIZE);
        user_adau1979_check_adi_result(result);
    }

    /* submit buffer2 */
    p_nn = ll_mgr_get_node(0);
    if (p_nn)
    {
        ll_mgr_insert(&adc_ll, p_nn);
        p_nn->size = USER_ADAU1979_1962A_BUFFER_SIZE;
        result = adi_adau1979_SubmitBuffer(phAdau1979, &p_nn->data, USER_ADAU1979_1962A_BUFFER_SIZE);
        user_adau1979_check_adi_result(result);
    }
#else
    p_nn = ll_mgr_get_node(0);
    if (p_nn)
    {
        ll_mgr_insert(&adc_ll, p_nn);
        p_nn->size = USER_ADAU1979_1962A_BUFFER_SIZE-(AUDIO_SAMPLE_SIZE_BYTES*NUM_AUDIO_CHANNELS);
        result = adi_adau1979_SubmitBuffer(phAdau1979, &p_nn->data, USER_ADAU1979_1962A_BUFFER_SIZE-(AUDIO_SAMPLE_SIZE_BYTES*NUM_AUDIO_CHANNELS));
        user_adau1979_check_adi_result(result);
    }

    /* submit buffer2 */
    p_nn = ll_mgr_get_node(0);
    if (p_nn)
    {
        ll_mgr_insert(&adc_ll, p_nn);
        p_nn->size = USER_ADAU1979_1962A_BUFFER_SIZE-(AUDIO_SAMPLE_SIZE_BYTES*NUM_AUDIO_CHANNELS);
        result = adi_adau1979_SubmitBuffer(phAdau1979, &p_nn->data, USER_ADAU1979_1962A_BUFFER_SIZE-(AUDIO_SAMPLE_SIZE_BYTES*NUM_AUDIO_CHANNELS));
        user_adau1979_check_adi_result(result);
    }
#endif
}

/**
 * Submits the ADAU1962A ping pong buffers to the ADI Sport System Service.
 *
 * @note
 * This function was copied, and then modified, from the ADI
 * examples provided with the SC589 EZ-Board Board Support Package.
 *
 * @retval 0 if success
 * @retval other values for error
 */
static void Adau1962aSubmitBuffers(void)
{
    LL_Mgr_Node * p_nn;
    ADI_ADAU1962A_RESULT result;

    /* submit buffer1 */
#if (USER_AUDIO_48kHz_SAMPLE_RATE)
    p_nn = ll_mgr_get_node(CLD_TRUE);
    if (p_nn)
    {
        ll_mgr_insert(&dac_ll, p_nn);
        p_nn->size = USER_ADAU1979_1962A_BUFFER_SIZE;
        result = adi_adau1962a_SubmitBuffer(phAdau1962a, &p_nn->data, USER_ADAU1979_1962A_BUFFER_SIZE);
        user_adau1962a_check_adi_result(result);
    }

    /* submit buffer2 */
    p_nn = ll_mgr_get_node(CLD_TRUE);
    if (p_nn)
    {
        ll_mgr_insert(&dac_ll, p_nn);
        p_nn->size = USER_ADAU1979_1962A_BUFFER_SIZE;
        result = adi_adau1962a_SubmitBuffer(phAdau1962a, &p_nn->data, USER_ADAU1979_1962A_BUFFER_SIZE);
        user_adau1962a_check_adi_result(result);
    }
#else
    p_nn = ll_mgr_get_node(CLD_TRUE);
    if (p_nn)
    {
        ll_mgr_insert(&dac_ll, p_nn);
        p_nn->size = USER_ADAU1979_1962A_BUFFER_SIZE-(AUDIO_SAMPLE_SIZE_BYTES*NUM_AUDIO_CHANNELS);
        result = adi_adau1962a_SubmitBuffer(phAdau1962a, &p_nn->data, USER_ADAU1979_1962A_BUFFER_SIZE-(AUDIO_SAMPLE_SIZE_BYTES*NUM_AUDIO_CHANNELS));
        user_adau1962a_check_adi_result(result);
    }

    /* submit buffer2 */
    p_nn = ll_mgr_get_node(CLD_TRUE);
    if (p_nn)
    {
        ll_mgr_insert(&dac_ll, p_nn);
        p_nn->size = USER_ADAU1979_1962A_BUFFER_SIZE-(AUDIO_SAMPLE_SIZE_BYTES*NUM_AUDIO_CHANNELS);
        result = adi_adau1962a_SubmitBuffer(phAdau1962a, &p_nn->data, USER_ADAU1979_1962A_BUFFER_SIZE-(AUDIO_SAMPLE_SIZE_BYTES*NUM_AUDIO_CHANNELS));
        user_adau1962a_check_adi_result(result);
    }
#endif
}

/**
 * Opens and initializes System Protection Unit peripheral.
 *
 * This function was copied, and then modified, from the ADI
 * examples provided with the SC589 EZ-Board Board Support Package.
 *
 * @retval 0 if success
 * @retval ADI_SPU_FAILURE - error
 */
static unsigned long SpuInit(void)
{
    unsigned long Result = 0u;

#if defined(__ADSPBF707_FAMILY__) || defined(__ADSPSC589_FAMILY__)
    /* Initialize SPU Service */
    if(adi_spu_Init(0u, SpuMemory, NULL, NULL, &hSpu) != ADI_SPU_SUCCESS)
    {
        printf("Failed to initialize SPU service\n\r");
        return (ADI_SPU_FAILURE);
    }

    /* Make SPORT 4A to generate secure transactions */
    if(adi_spu_EnableMasterSecure(hSpu, SPORT_4A_SPU_PID, true) != ADI_SPU_SUCCESS)
    {
        printf("Failed to enable Master secure for SPORT 4A\n\r");
        return (ADI_SPU_FAILURE);
    }

    /* Make SPORT 4B to generate secure transactions */
    if(adi_spu_EnableMasterSecure(hSpu, SPORT_4B_SPU_PID, true) != ADI_SPU_SUCCESS)
    {
        printf("Failed to enable Master secure for SPORT 4B\n\r");
        return (ADI_SPU_FAILURE);
    }

    /* Make SPORT 4A DMA to generate secure transactions */
    if(adi_spu_EnableMasterSecure(hSpu, SPORT_4A_DMA10_SPU_PID, true) != ADI_SPU_SUCCESS)
    {
        printf("Failed to enable Master secure for SPORT 4A DMA\n\r");
        return (ADI_SPU_FAILURE);
    }

    /* Make SPORT 4B DMA to generate secure transactions */
    if(adi_spu_EnableMasterSecure(hSpu, SPORT_4B_DMA11_SPU_PID, true) != ADI_SPU_SUCCESS)
    {
        printf("Failed to enable Master secure for SPORT 4B DMA\n\r");
        return (ADI_SPU_FAILURE);
    }
#elif defined(__ADSPSC573_FAMILY__)

    /* Initialize SPU Service */
    if(adi_spu_Init(0u, SpuMemory, NULL, NULL, &hSpu) != ADI_SPU_SUCCESS)
    {
        printf("Failed to initialize SPU service\n\r");
        return (ADI_SPU_FAILURE);
    }

    /* Make SPORT 2A to generate secure transactions */
    if(adi_spu_EnableMasterSecure(hSpu, SPORT_2A_SPU_PID, true) != ADI_SPU_SUCCESS)
    {
        printf("Failed to enable Master secure for SPORT 2A\n\r");
        return (ADI_SPU_FAILURE);
    }

    /* Make SPORT 2B to generate secure transactions */
    if(adi_spu_EnableMasterSecure(hSpu, SPORT_2B_SPU_PID, true) != ADI_SPU_SUCCESS)
    {
        printf("Failed to enable Master secure for SPORT 2B\n\r");
        return (ADI_SPU_FAILURE);
    }

    /* Make SPORT 2A DMA to generate secure transactions */
    if(adi_spu_EnableMasterSecure(hSpu, SPORT_2A_DMA4_SPU_PID, true) != ADI_SPU_SUCCESS)
    {
        printf("Failed to enable Master secure for SPORT 2A DMA\n\r");
        return (ADI_SPU_FAILURE);
    }

    /* Make SPORT 2B DMA to generate secure transactions */
    if(adi_spu_EnableMasterSecure(hSpu, SPORT_2B_DMA5_SPU_PID, true) != ADI_SPU_SUCCESS)
    {
        printf("Failed to enable Master secure for SPORT 2B DMA\n\r");
        return (ADI_SPU_FAILURE);
    }

#else
#error "processor not supported"
#endif

    return (Result);
}

/**
 * Determines if additional DAC data is needed from the Host.
 *
 * @retval The frequency in kHz needed to maintain the specified sample rate (48 or 44.1kHz
 */
float user_adau1962a_get_feedback_value (void)
{
#if (USER_AUDIO_48kHz_SAMPLE_RATE)
    #if (USER_AUDIO_VARIABLE_FEEDBACK)
    if (user_audio_w_cdc_get_audio_out_data_num_elements() <= 1)
    {
        return 49;
    }
    else
    {
        return 48;
    }
    #else
    return 48;
    #endif
#else
    #if (USER_AUDIO_VARIABLE_FEEDBACK)
    if (user_audio_w_cdc_get_audio_out_data_num_elements() <= 1)
    {
        return 45.1;
    }
    else
    {
        return 44.1;
    }
    #else
    return 44.1;
    #endif
#endif
}
